@@ -35,8 +35,7 @@ holds the key and hash value.
 #define PERL_HASH_INTERNAL_ACCESS
 #include "perl.h"
 
-#define HV_MAX_LENGTH_BEFORE_REHASH 14
-#define SHOULD_DO_HSPLIT(xhv) ((xhv)->xhv_keys > (xhv)->xhv_max) /* HvTOTALKEYS(hv) > HvMAX(hv) */
+#define HV_MAX_LENGTH_BEFORE_SPLIT 14
 
 static const char S_strtab_error[]
     = "Cannot modify shared string table in hv_%s";
@@ -278,7 +277,10 @@ negative the key is assumed to be in UTF-8-encoded Unicode.
 Returns the SV which corresponds to the specified key in the hash.
 The absolute value of C<klen> is the length of the key.  If C<klen> is
 negative the key is assumed to be in UTF-8-encoded Unicode.  If
-C<lval> is set then the fetch will be part of a store.  Check that the
+C<lval> is set then the fetch will be part of a store.  This means that if
+there is no value in the hash associated with the given key, then one is
+created and a pointer to it is returned.  The C<SV*> it points to can be
+assigned to.  But always check that the
 return value is non-null before dereferencing it to an C<SV*>.
 
 See L<perlguts/"Understanding the Magic of Tied Hashes and Arrays"> for more
@@ -799,9 +801,29 @@ Perl_hv_common(pTHX_ HV *hv, SV *keysv, const char *key, STRLEN klen,
     if (masked_flags & HVhek_ENABLEHVKFLAGS)
 	HvHASKFLAGS_on(hv);
 
-    xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
-    if ( SHOULD_DO_HSPLIT(xhv) ) {
-        hsplit(hv);
+    {
+	const HE *counter = HeNEXT(entry);
+
+	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
+	if (!counter) {				/* initial entry? */
+	} else if (xhv->xhv_keys > xhv->xhv_max) {
+		/* Use only the old HvUSEDKEYS(hv) > HvMAX(hv) condition to limit
+		   bucket splits on a rehashed hash, as we're not going to
+		   split it again, and if someone is lucky (evil) enough to
+		   get all the keys in one list they could exhaust our memory
+		   as we repeatedly double the number of buckets on every
+		   entry. Linear search feels a less worse thing to do.  */
+	    hsplit(hv);
+	} else if(!HvREHASH(hv)) {
+	    U32 n_links = 1;
+
+	    while ((counter = HeNEXT(counter)))
+		n_links++;
+
+	    if (n_links > HV_MAX_LENGTH_BEFORE_SPLIT) {
+		hsplit(hv);
+	    }
+	}
     }
 
     if (return_svp) {
@@ -1178,7 +1200,7 @@ S_hsplit(pTHX_ HV *hv)
 
 
     /* Pick your policy for "hashing isn't working" here:  */
-    if (longest_chain <= HV_MAX_LENGTH_BEFORE_REHASH /* split worked?  */
+    if (longest_chain <= HV_MAX_LENGTH_BEFORE_SPLIT /* split worked?  */
 	|| HvREHASH(hv)) {
 	return;
     }
@@ -2763,8 +2785,8 @@ S_share_hek_flags(pTHX_ const char *str, I32 len, register U32 hash, int flags)
 
 	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
 	if (!next) {			/* initial entry? */
-	} else if ( SHOULD_DO_HSPLIT(xhv) ) {
-            hsplit(PL_strtab);
+	} else if (xhv->xhv_keys > xhv->xhv_max /* HvUSEDKEYS(hv) > HvMAX(hv) */) {
+		hsplit(PL_strtab);
 	}
     }
 
@@ -3494,8 +3516,8 @@ Perl_hv_assert(pTHX_ HV *hv)
  * Local variables:
  * c-indentation-style: bsd
  * c-basic-offset: 4
- * indent-tabs-mode: t
+ * indent-tabs-mode: nil
  * End:
  *
- * ex: set ts=8 sts=4 sw=4 noet:
+ * ex: set ts=8 sts=4 sw=4 et:
  */
